#include <unistd.h>
#include <sys/wait.h>
#include <string.h>
#include <errno.h>
#include "misc.h"

void fork_procs (n)
	unsigned long n;
{
	unsigned long i;
	long pid;
	lx_s ns = {0};
	lx_s nt = {0};

	for (i = 0; i++ < n;) {
		pid = (long)fork();
		if (pid < 0) {
			lx_straddulong (&ns, i--, 10);
			lx_check0 (&ns);
			outlog (gderr, "couldn't fork process ", ns.s, ": ",
					strerror(errno), 0);
			lx_free (&ns);
			continue;
		}
		
		/* if we're the child, continue with life */
		if (!pid) return;

		lx_straddulong (&ns, i, 10);
		lx_straddulong (&nt, pid, 10);
		lx_check0 (&ns);
		lx_check0 (&nt);
		debuglog (1, gderr, "forked process ", ns.s, " (", nt.s, ")");
		lx_free (&ns);
		lx_free (&nt);
	}

	debuglog (1, gderr, "all processes forked; waiting...", 0,0,0,0);

//	while ((pid = (long)waitpid(-1, 0, WNOHANG)) > 0 || errno == EINTR) {
	while ((pid = (long)wait(0)) > 0 /* || errno == EINTR */) {
//		if (errno == EINTR) { errno = 0; continue; }
		ns.len = nt.len = 0;
		lx_straddulong (&ns, --i, 10);
		lx_straddulong (&nt, pid, 10);
		lx_check0 (&ns);
		lx_check0 (&nt);
		debuglog (1, gderr, "process ", ns.s, " exited (", nt.s, ")");
	}

	debuglog (1, gderr, "all processes exited.", 0,0,0,0);
	_exit(0);
}

