#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "misc.h"

void writeready();

struct timeval fullstart, fullend; /* time for all connections */
struct timeval connstart, connend; /* time for full connection */
struct timeval sendstart, sendend; /* time for sending */
struct timeval readstart, readend; /* time for reading */

char s_pid[40]; /* enough for a 128-bit pid */

void loadtest(host)
	char *host;
{
	opt_ulong_t i;
	int sd;
	static lx_gd _gdsd;
	lx_gd *gdsd = &_gdsd; /* we'll use this for reading _and_ writing */

	unsigned long rbytes; /* bytes read */
	unsigned long sbytes; /* bytes sent */

	/* YES, sprintf is FINE here; don't complain. */
	sprintf (s_pid, "%ld", (long)getpid()); /* get pid */

	if (opt_F.v_opt.opt_ulong > 1)
		fork_procs(opt_F.v_opt.opt_ulong);

	sprintf (s_pid, "%ld", (long)getpid()); /* again, in case we're a child */

	/* do this now, to save time in the for(;;) loop */
	if (lx_gdnew (gdsd, 0, opt_x.v_opt.opt_ulong))
		die_nomem();
	/* NOTE: if opt_x is small, a lot of writes will be used
	 * for send_cmds().  This shouldn't matter, and may even be
	 * somewhat desirable (emulate some modem speed while sending too).
	 * XXX: perhaps we should include sleeps in send_cmds() as well?
	 */

	gettimeofday (&fullstart, 0);
	/* connect opt_n times */
	for (i = 0; i++ < opt_n.v_opt.opt_ulong;) {
		if (opt_t.v_opt.opt_string)
			wait_file (opt_t.v_opt.opt_string);
		gettimeofday (&connstart, 0);
		sd = doconnect (host);
		writeready(gdsd, sd);

		gettimeofday (&sendstart, 0);
		if (send_cmds (gdsd, &sbytes)) {
			outlog (gderr, "write error: ", strerror(errno), 0,0,0);
		} else {
			gettimeofday (&sendend, 0);
			if (opt_ts.v_opt.opt_bool)
				pntime ("send", &sendstart, &sendend, i);
			gettimeofday (&readstart, 0);
			read_output (gdsd, &rbytes);
			gettimeofday (&readend, 0);
			if (opt_tr.v_opt.opt_bool)
				pntime ("read", &readstart, &readend, i);
		}
		close(sd);
		gettimeofday (&connend, 0);
		if (opt_tf.v_opt.opt_bool)
			pntime ("full connection", &connstart, &connend, i);
		nanosleepy (&time_N);
	}
	gettimeofday (&fullend, 0);
	if (opt_ta.v_opt.opt_bool)
		ptime ("all connections", &fullstart, &fullend);
}

void writeready (gd, fd)
	lx_gd *gd;
	int fd;
{
	gd->fd = fd;
	LX_gdrw(gd);
}
